// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System.Collections.Specialized;
using System.Text;

namespace EonaCat.Network
{
    /// <summary>
    /// Represents a collection of query string parameters in the EonaCat network library.
    /// </summary>
    internal sealed class QueryStringCollection : NameValueCollection
    {
        /// <summary>
        /// Converts the collection to its string representation.
        /// </summary>
        /// <returns>A string representation of the query string parameters.</returns>
        public override string ToString()
        {
            var count = Count;
            if (count == 0)
            {
                return string.Empty;
            }

            var output = new StringBuilder();
            var keys = AllKeys;
            foreach (var key in keys)
            {
                output.AppendFormat($"{key}={this[key]}&");
            }

            if (output.Length > 0)
            {
                output.Length--;
            }

            return output.ToString();
        }
    }
}