// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;
using System.Threading;

namespace EonaCat.Network
{
    internal class HttpStreamAsyncResult : IAsyncResult
    {
        private readonly AsyncCallback _callback;
        private bool _isCompleted;
        private readonly object _sync;
        private ManualResetEvent _waitHandle;

        internal HttpStreamAsyncResult(AsyncCallback callback, object state)
        {
            _callback = callback;
            AsyncState = state;
            _sync = new object();
        }

        internal byte[] Buffer { get; set; }

        internal int Count { get; set; }

        internal Exception Exception { get; private set; }

        internal bool HasException => Exception != null;

        internal int Offset { get; set; }

        internal int SyncRead { get; set; }

        public object AsyncState { get; }

        public WaitHandle AsyncWaitHandle
        {
            get
            {
                lock (_sync)
                {
                    return _waitHandle ??= new ManualResetEvent(_isCompleted);
                }
            }
        }

        public bool CompletedSynchronously => SyncRead == Count;

        public bool IsCompleted
        {
            get
            {
                lock (_sync)
                {
                    return _isCompleted;
                }
            }
        }

        internal void Complete()
        {
            lock (_sync)
            {
                if (_isCompleted)
                {
                    return;
                }

                _isCompleted = true;

                _waitHandle?.Set();

                _callback?.Invoke(this);
            }
        }

        internal void Complete(Exception exception)
        {
            Exception = exception;
            Complete();
        }
    }
}