﻿// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;
using System.Net.Security;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;

namespace EonaCat.Network
{
    public class SSLConfigClient
    {
        private LocalCertificateSelectionCallback _clientCertSelectionCallback;
        private X509CertificateCollection _clientCertificates;
        private RemoteCertificateValidationCallback _serverCertValidationCallback;

        public SSLConfigClient()
        {
            SslProtocols = SslProtocols.Tls12;
        }

        public SSLConfigClient(string targetHost)
        {
            TargetHost = targetHost;
            SslProtocols = SslProtocols.Tls12;
        }

        public SSLConfigClient(SSLConfigClient configuration)
        {
            if (configuration == null)
            {
                throw new ArgumentNullException(nameof(configuration));
            }

            CheckForCertificateRevocation = configuration.CheckForCertificateRevocation;
            _clientCertSelectionCallback = configuration._clientCertSelectionCallback;
            _clientCertificates = configuration._clientCertificates;
            SslProtocols = configuration.SslProtocols;
            _serverCertValidationCallback = configuration._serverCertValidationCallback;
            TargetHost = configuration.TargetHost;
        }

        public bool CheckForCertificateRevocation { get; set; }

        public X509CertificateCollection Certificates
        {
            get
            {
                _clientCertificates ??= new X509CertificateCollection();
                return _clientCertificates;
            }

            set
            {
                _clientCertificates = value;
            }
        }

        public LocalCertificateSelectionCallback ClientCertificateSelectionCallback
        {
            get
            {
                _clientCertSelectionCallback ??= SelectClientCertificate;

                return _clientCertSelectionCallback;
            }

            set
            {
                _clientCertSelectionCallback = value;
            }
        }

        public SslProtocols SslProtocols { get; set; }

        public RemoteCertificateValidationCallback ServerCertificateValidationCallback
        {
            get
            {
                _serverCertValidationCallback ??= ValidateServerCertificate;

                return _serverCertValidationCallback;
            }

            set
            {
                _serverCertValidationCallback = value;
            }
        }

        public string TargetHost { get; set; }

        private static X509Certificate SelectClientCertificate(
          object sender,
          string targetHost,
          X509CertificateCollection clientCertificates,
          X509Certificate serverCertificate,
          string[] acceptableIssuers
        )
        {
            return null;
        }

        private static bool ValidateServerCertificate(
          object sender,
          X509Certificate certificate,
          X509Chain chain,
          SslPolicyErrors sslPolicyErrors
        )
        {
            return true;
        }
    }
}