﻿// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;
using System.Net.Security;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;

namespace EonaCat.Network
{
    public class SSLConfigServer
    {
        private RemoteCertificateValidationCallback _clientCertificationValidationCallback;

        public SSLConfigServer()
        {
            SslProtocols = SslProtocols.Tls12;
        }

        public SSLConfigServer(X509Certificate2 certificate)
        {
            Certificate = certificate;
            SslProtocols = SslProtocols.Tls12;
        }

        public SSLConfigServer(SSLConfigServer configuration)
        {
            if (configuration == null)
            {
                throw new ArgumentNullException(nameof(configuration));
            }

            CheckForCertificateRevocation = configuration.CheckForCertificateRevocation;
            IsClientCertificateRequired = configuration.IsClientCertificateRequired;
            _clientCertificationValidationCallback = configuration._clientCertificationValidationCallback;
            SslProtocols = configuration.SslProtocols;
            Certificate = configuration.Certificate;
        }

        public bool CheckForCertificateRevocation { get; set; }

        public bool IsClientCertificateRequired { get; set; }

        public RemoteCertificateValidationCallback ClientCertificateValidationCallback
        {
            get
            {
                _clientCertificationValidationCallback ??= ValidateClientCertificate;

                return _clientCertificationValidationCallback;
            }

            set
            {
                _clientCertificationValidationCallback = value;
            }
        }

        public SslProtocols SslProtocols { get; set; }

        public X509Certificate2 Certificate { get; set; }

        private static bool ValidateClientCertificate(
          object sender,
          X509Certificate certificate,
          X509Chain chain,
          SslPolicyErrors sslPolicyErrors
        )
        {
            return true;
        }
    }
}